"""VeraCore REST API Client Package"""
# flake8: noqa

from .api import VeraCore
from .exceptions import \
        VeraCoreRequestForbidden, \
        VeraCoreRequestError, \
        VeraCoreNoSession



""" VeraCore API Constants """


# ORDER STATUSES
ORDER_STATUS_UNPROCESSED = 'Unprocessed'
ORDER_STATUS_PROCESSED = 'Processed'
ORDER_STATUS_PARTIAL_SHIP = 'PartiallyShipped'
ORDER_STATUS_COMPLETED = 'Complete'
ORDER_STATUS_CANCELLED = 'Cancelled'
ORDER_STATUS_ALL = 'All'

ORDER_STATUSES = [
    ORDER_STATUS_UNPROCESSED,
    ORDER_STATUS_PROCESSED,
    ORDER_STATUS_PARTIAL_SHIP,
    ORDER_STATUS_COMPLETED,
    ORDER_STATUS_CANCELLED,
    ORDER_STATUS_ALL
]


# SHIPPING LABEL STATUSES
SHIPPING_LABEL_NONE = 'None'
SHIPPING_LABEL_EXISTS = 'Exists'
SHIPPING_LABEL_EXCEPT = 'Exception'

SHIPPING_LABEL_STATUSES = [
    SHIPPING_LABEL_NONE,
    SHIPPING_LABEL_EXISTS,
    SHIPPING_LABEL_EXCEPT
]



""" All exceptions for VeraCore REST API Client """


import logging


# pylint: disable=invalid-name
logger = logging.getLogger(__name__)


class VeraCoreException(Exception):
    """Base VeraCore API exception"""

    message = 'Unknown error occurred for {url}. Response content: {content}'

    def __init__(self, url, status, resource_name, content):
        """Initialize the VeraCoreException class
        VeraCoreException is the base class exception for all exceptions
        generated by this API client

        Arguments:
            url: VeraCore URL that was called
            status: Status code of the error response
            resource_name: Name of the VeraCore resource being queried
            content: content of the response
        """
        # pylint: disable=super-init-not-called
        self.url = url
        self.status = status
        self.resource_name = resource_name
        self.content = content

        if '{status}' in self.message:
            self.log_message = self.message.format(
                url=self.url, status=self.status, content=self.content)
        else:
            self.log_message = self.message.format(
                url=self.url, content=self.content)

        logger.error(self.log_message)

    def __str__(self):
        return self.message.format(url=self.url, content=self.content)


class VeraCoreRequestForbidden(VeraCoreException):
    """
    Error Code: 403
    The request has generated a forbidden access error.
    Verify that the service account has authenticated with a valid
    username, password, & system ID. Also verify that the token
    has not expired.
    """

    message = 'Request refused for {url}. Response content: {content}'


class VeraCoreRequestError(VeraCoreException):
    """
    Error Code: Dynamic
    The request completed with an error.
    """

    message = 'Request to {url} has failed with status {status}. {content}'

    def __str__(self):
        return self.message.format(
            url=self.url, status=self.status, content=self.content)


class VeraCoreNoSession(VeraCoreException):
    """
    Error Code: None
    A request was about to be attempted that didn't have a valid session.
    Try reinitiallizing or calling the login() method.
    """

    # pylint: disable=line-too-long
    message = 'No valid authentication token exists when attempting to call {url}. {content}'



"""
Version details for VeraCore API Client
"""

__title__ = 'veracore_api_client'
__description__ = 'A simple class-based Python client for getting data \
                  from the VeraCore REST API'
__url__ = 'https://github.com/veracore-api/veracore-api-client-python'
__version__ = '0.0.3.alpha2'
__author__ = 'Eli Keimig'
__author_email__ = 'ewkeimig@cyclops26.com'
__license__ = 'GNU General Public License v3 or later (GPLv3+)'
__maintainer__ = 'Eli Keimig'
__maintainer_email__ = 'ewkeimig@cyclops26.com'
__keywords__ = 'python veracore oms wms veracore.com'


def get_version_information(display=True, debug=False):
    """ prints the package version information """
    # pylint: disable=line-too-long
    version = 'VeraCore API Python Client: Version {version}. Released under a {license} license. Author: {author}.'.format(
                  version=__version__, license=__license__, author=__author__
              )
    if display:
        print(version)
    return version if not display or debug else None



"""Core class for VeraCore"""

DEFAULT_API_VERSION = '2.0'

import datetime
import logging

import requests

from .constants import \
        ORDER_STATUSES, \
        SHIPPING_LABEL_STATUSES

from .exceptions import \
        VeraCoreRequestForbidden, \
        VeraCoreNoSession, \
        VeraCoreRequestError


# pylint: disable=invalid-name
logger = logging.getLogger(__name__)


# pylint: disable=too-many-instance-attributes
class VeraCore:
    """ VeraCore API Client Class """

    # pylint: disable=too-many-arguments,too-many-locals,too-many-branches
    def __init__(
            self,
            username,
            password,
            system_id,
            domain,
            session=None,
            authentication_token=None,
            version=DEFAULT_API_VERSION,
        ):
        """ Initialize the instance with the given parameters. """

        self.username = username
        self.password = password
        self.system_id = system_id
        self.domain = domain
        self.session = session or requests.Session()
        self.authentication_token = authentication_token
        self.authentication_token_expiration = None
        self.api_version = version

        self.login()

    def login(self):
        """ authenticate the instance for future requests """
        url = 'https://%s/VeraCore/Public.Api/api/login' % self.domain
        request = self.session.post(
            url, json={
                "userName": self.username,
                "password": self.password,
                "systemId": self.system_id
            })

        data = request.json()

        if request.status_code == 403:
            raise VeraCoreRequestForbidden(
                url, request.status_code, 'Login',
                data.get('Error', 'Unknown') if data else 'Unknown')
        elif request.status_code != 200:
            raise VeraCoreRequestError(
                url, request.status_code, 'Login',
                data.get('Error', 'Unknown') if data else 'Unknown')
        else:
            self.authentication_token = data.get('Token', None)
            self.authentication_token_expiration = datetime.datetime.strptime(
                data.get('UtcExpirationDate').split('.')[0],
                '%Y-%m-%dT%H:%M:%S') \
                    if data.get('UtcExpirationDate', None) else None

    def check_authentication_token(self):
        """Verify that a authentication token exists and has not expired"""
        return bool(
            self.authentication_token and \
                self.authentication_token_expiration > datetime.datetime.now())

    def get_orders(
            self,
            order_status=None,
            shipping_label_status=None,
            single_piece=None,
            carrier_code=None,
            stream=None,
            stream_start=None,
            stream_end=None,
            ):
        """ get orders matching the specified criteria """

        url = 'https://%s/VeraCore/Public.Api/api/orders?api=True' % self.domain

        if order_status and order_status in ORDER_STATUSES:
            url = '%s&request.status=%s' % (url, order_status)

        if shipping_label_status and \
                shipping_label_status in SHIPPING_LABEL_STATUSES:
            url = '%s&request.shippingLabelStatus=%s' % (
                url, shipping_label_status)

        if single_piece and isinstance(single_piece, bool):
            url = '%s&request.singlePiece=%s' % (url, single_piece)

        if carrier_code and isinstance(carrier_code, str):
            url = '%s&request.carrierCode=%s' % (url, carrier_code)

        if stream and isinstance(stream, str):
            url = '%s&request.carrierCode=%s' % (url, stream)

        if stream_start and isinstance(stream_start, datetime.datetime):
            url = '%s&request.streamAssignedUTCStartDate=%s' % (
                url,
                datetime.datetime.strftime(stream_start, '%Y-%m-%dT%H:%M:%S'))

        if stream_end and isinstance(stream_end, datetime.datetime):
            url = '%s&request.streamAssignedUTCEndDate=%s' % (
                url,
                datetime.datetime.strftime(
                    stream_end, '%Y-%m-%dT%H:%M:%S'))

        if not self.authentication_token or \
                not self.authentication_token_expiration or \
                datetime.datetime.now() > self.authentication_token_expiration:
            raise VeraCoreNoSession(url, None, 'Orders', '')

        request = self.session.get(
            url,
            headers={
                'Authorization': 'Bearer %s' % \
                    self.authentication_token
            })

        data = request.json()

        if request.status_code == 403:
            raise VeraCoreRequestForbidden(
                url, request.status_code, 'Orders',
                data.get('Error', 'Unknown') if data else 'Unknown')
        elif request.status_code != 200:
            raise VeraCoreRequestError(
                url, request.status_code, 'Orders',
                data.get('Error', 'Unknown') if data else 'Unknown')
        else:
            return data.get('Orders', [])



"""VeraCore API Client Package Setup"""

import os
import textwrap

from setuptools import setup

here = os.path.abspath(os.path.dirname(__file__))

about = {}
with open(os.path.join(here, 'veracore_api_client', '__version__.py'), 'r') as f:
    exec(f.read(), about)

setup(
    name=about['__title__'],
    version=about['__version__'],
    author=about['__author__'],
    author_email=about['__author_email__'],
    maintainer=about['__maintainer__'],
    maintainer_email=about['__maintainer_email__'],
    packages=['veracore_api_client',],
    url=about['__url__'],
    license=about['__license__'],
    description=about['__description__'],
    long_description=textwrap.dedent(open('README.rst', 'r').read()),
    long_description_content_type='text/x-rst',
    install_requires=[
        'requests>=2.22.0'
    ],
    tests_require=[
        'nose>=1.3.0',
        'pytz>=2014.1.1',
        'responses>=0.5.1',
    ],
    test_suite='nose.collector',
    keywords=about['__keywords__'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Operating System :: OS Independent',
        'Topic :: Internet :: WWW/HTTP',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9'
    ]
)